"""Rich HTML dashboard package for comprehensive system monitoring."""

from .html_dashboard import ComprehensiveHTMLDashboard, DashboardAssetManager

__all__ = [
    "ComprehensiveHTMLDashboard",
    "DashboardAssetManager",
]
